#pragma once
/****************************************************************************
*                                                                           *
* Copyright(C) 2007                                                         *
* Pierluigi Failla                                                          *
*																			*
* http://www.zeta.xm.com													*
*                                                                           *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *   
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/
#include <stdio.h>
#include <windows.h>
#include <math.h>

#include <GL/glew.h>

#include "ParticleEngine.h"
/*
Enjoy my Synaptic Encreasing Particle Engine, and remember:
one point is nothing...
billion of point are a Particles System!!! (PF)
*/

/*
Point3D
*/
Point3D::Point3D(void){
	x = 0.0;
	y = 0.0;
	z = 0.0;
}

Point3D::Point3D(float _x, float _y, float _z){
	x = _x;
	y = _y;
	z = _z;
}


/*
Particle
*/
Particle::Particle(void){
	Life = 1.0;
	Fade = 0.5;
	Speed.x = 1.0;
	Speed.y = 1.0;
	Speed.z = 1.0;
	Angle.x = 0.0;
	Angle.y = 0.0;
	Angle.z = 0.0;
}
/*
Particles Turbo Mega System!!!!!!!!!!!!!!!!!!!!
*/
Particles::Particles(void){
	MAX_PARTICLES = 500;
	SysPa = new Particle[MAX_PARTICLES];
	SlowDownFactor = 0.99;
	SpreadFactor = 1.0;
	Distrib3D = true;
}
Particles::Particles(int TotParticles){
	MAX_PARTICLES = TotParticles;
	SysPa = new Particle[MAX_PARTICLES];
	SlowDownFactor = 0.99;
	SpreadFactor = 1.0;
	Distrib3D = true;
}
bool Particles::SetTexture(GLuint TextureNumber){
	int texWidth = 256, texHeight = 256; //texture 256x256
    GLubyte *texPixels, *p;
    int texSize, radius, i, j;

    texSize = texWidth*texHeight*4*sizeof(GLubyte);
    texPixels = (GLubyte *) malloc(texSize);
    
	if (texPixels == NULL){return false;}

    p = texPixels;
	for (i=0; i<texHeight; ++i){
		for (j=0; j<texWidth; ++j){
			GLuint dist = hypot(float(i - (texHeight / 2)),float(j - (texWidth / 2)));
            float color = 255-(dist*1.8);
			if (color < 0.0){color = 0.0;}
			p[0] = color;
			p[1] = color;
			p[2] = color;
			p[3] = color;   
			p+=4;
		}
    }

	Tex = TextureNumber;
	glBindTexture( GL_TEXTURE_2D, TextureNumber );
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, 3, texWidth, texHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, texPixels);

    free(texPixels);
    return true;
}
void Particles::UseThisTexture(GLuint TextureNumber){
	Tex = TextureNumber;
}
void Particles::InitSystem(void){
	for(int i=0; i<MAX_PARTICLES; i++){
		SysPa[i].Life = (float)1.0; //complete life 1.0
		SysPa[i].Fade = (float)((rand()%MAX_PARTICLES)/MAX_PARTICLES)+0.05; //fade random speed
		PositionRule(i);
		SpeedRule(i); //speed
		AngleRule(i); //angle
		DirectionRule(i); //direction
	}
}
void Particles::DrawSystem(void){
	glDisable(GL_LIGHTING);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glEnable(GL_ALPHA_TEST);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
	glBindTexture(GL_TEXTURE_2D, Tex);

	for(int i=0; i<MAX_PARTICLES; i++){
		float x = SysPa[i].Position.x;
		float y = SysPa[i].Position.y;
		float z = SysPa[i].Position.z;

		//glColor4f(0.5,0.5,1.0,SysPa[i].Life); //fade based on life
		//glColor4f(abs(x/(x+y+z)),abs(y/(x+y+z)),abs(z/(x+y+z)),SysPa[i].Life);
		glColor4f(1,1-SysPa[i].Life,0,SysPa[i].Life);

		if(Distrib3D){
			int ang = ((int)(SysPa[i].Angle.x + SysPa[i].Angle.y + SysPa[i].Angle.z))%360;
			glPushMatrix();
				glRotatef(ang,x,y,z);
		}

		glBegin(GL_TRIANGLE_STRIP); // Build Quad From A Triangle Strip
			glTexCoord2f(1,1); glVertex3f(x+0.05,y+0.05,z-0.05); // Top Right
			glTexCoord2f(0,1); glVertex3f(x-0.05,y+0.05,z+0.05); // Bottom Right
			glTexCoord2f(1,0); glVertex3f(x+0.05,y-0.05,z-0.05); // Top Left
			glTexCoord2f(0,0); glVertex3f(x-0.05,y-0.05,z+0.05); // Bottom Left
		glEnd(); // Done Building Triangle Strip
		
		if(Distrib3D){
			glPopMatrix();
		}
		
		SysPa[i].Position.x += SysPa[i].Direction.x * SysPa[i].Fade;
		SysPa[i].Position.y += SysPa[i].Direction.y * SysPa[i].Fade;
		SysPa[i].Position.z += SysPa[i].Direction.z * SysPa[i].Fade;

		SysPa[i].Position.y = abs(SysPa[i].Position.y);//so it all up

		/*SysPa[i].Position.x = cos(SysPa[i].Position.x);
		SysPa[i].Position.y = abs(cos(SysPa[i].Position.y)*SysPa[i].Position.y);
		SysPa[i].Position.z = sin(SysPa[i].Position.z);*/

		SysPa[i].Direction.x *= SlowDownFactor - SysPa[i].Fade; //slow down
		SysPa[i].Direction.y *= SlowDownFactor - SysPa[i].Fade;
		SysPa[i].Direction.z *= SlowDownFactor - SysPa[i].Fade;

		SysPa[i].Life -= SysPa[i].Fade; //reduce life

		DirectionRule(i);
		CheckSpread_ABS(i);

		if(SysPa[i].Life < 0.05f){//quite dead!!!
			SysPa[i].Life = (float)1.0; //relive!!!!
			SysPa[i].Fade = (float)((rand()%MAX_PARTICLES)/MAX_PARTICLES) + 0.05;
			//PositionRule(i);
			SpeedRule(i);
			AngleRule(i);
		}
	}
	glDisable(GL_ALPHA_TEST);
	glDisable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);
	glEnable(GL_LIGHTING);
}
void Particles::DirectionRule(int ParticleNumber){
	SysPa[ParticleNumber].Direction.x = (sin(SysPa[ParticleNumber].Angle.x) + cos(SysPa[ParticleNumber].Angle.x)) * SysPa[ParticleNumber].Speed.x;
	SysPa[ParticleNumber].Direction.y = (sin(SysPa[ParticleNumber].Angle.y) + cos(SysPa[ParticleNumber].Angle.y)) * SysPa[ParticleNumber].Speed.y;
	SysPa[ParticleNumber].Direction.z = (sin(SysPa[ParticleNumber].Angle.z) + cos(SysPa[ParticleNumber].Angle.z)) * SysPa[ParticleNumber].Speed.z;
	SysPa[ParticleNumber].Angle.x++;
	SysPa[ParticleNumber].Angle.y++;
	SysPa[ParticleNumber].Angle.z++;
}
void Particles::SpeedRule(int ParticleNumber){
	SysPa[ParticleNumber].Speed.x = (float)(rand()%50);
	SysPa[ParticleNumber].Speed.y = (float)(rand()%50);
	SysPa[ParticleNumber].Speed.z = (float)(rand()%50);
}
void Particles::AngleRule(int ParticleNumber){
	SysPa[ParticleNumber].Angle.x = (float)(rand()%360);
	SysPa[ParticleNumber].Angle.y = (float)(rand()%360);
	SysPa[ParticleNumber].Angle.z = (float)(rand()%360);
}
void Particles::PositionRule(int ParticleNumber){
	SysPa[ParticleNumber].Position.x = (float)((rand()%50)/100)-0.5;
	SysPa[ParticleNumber].Position.y = (float)((rand()%50)/100)-0.5;
	SysPa[ParticleNumber].Position.z = (float)((rand()%50)/100)-0.5;
}
void Particles::CheckSpread_SQRT(int ParticleNumber){
	float s = 0.0;
	s = sqrt(SysPa[ParticleNumber].Position.x * SysPa[ParticleNumber].Position.x + SysPa[ParticleNumber].Position.y * SysPa[ParticleNumber].Position.y + SysPa[ParticleNumber].Position.z * SysPa[ParticleNumber].Position.z);
	if(s > SpreadFactor){
		SysPa[ParticleNumber].Life *= 0.7;
		SysPa[ParticleNumber].Speed.x *= 0.8;
		SysPa[ParticleNumber].Speed.y *= 0.8;
		SysPa[ParticleNumber].Speed.z *= 0.8;
	}
}
void Particles::CheckSpread_ABS(int ParticleNumber){
	if(abs(SysPa[ParticleNumber].Position.x) > SpreadFactor){SysPa[ParticleNumber].Position.x = 0.2*cos(SysPa[ParticleNumber].Position.x)*sin(SysPa[ParticleNumber].Position.x);}
	if(abs(SysPa[ParticleNumber].Position.y) > SpreadFactor){SysPa[ParticleNumber].Position.y = cos(SysPa[ParticleNumber].Position.y)*sin(SysPa[ParticleNumber].Position.y);}
	if(abs(SysPa[ParticleNumber].Position.z) > SpreadFactor){SysPa[ParticleNumber].Position.z = 0.2 *cos(SysPa[ParticleNumber].Position.z)*sin(SysPa[ParticleNumber].Position.z);}
}